@@ -412,20 +412,64 @@ def _conf_values_type_convert(conf):
     return opts
 
 
-class AuthProtocol(object):
+class _BaseAuthProtocol(object):
+    """A base class for AuthProtocol token checking implementations.
+
+    :param Callable app: The next application to call after middleware.
+    :param logging.Logger log: The logging object to use for output. By default
+                               it will use a logger in the
+                               keystonemiddleware.auth_token namespace.
+    """
+
+    def __init__(self, app, log=_LOG):
+        self.log = log
+        self._app = app
+
+    @webob.dec.wsgify(RequestClass=_request._AuthTokenRequest)
+    def __call__(self, req):
+        """Handle incoming request."""
+        response = self.process_request(req)
+        if response:
+            return response
+        response = req.get_response(self._app)
+        return self.process_response(response)
+
+    def process_request(self, request):
+        """Process request.
+
+        If this method returns a value then that value will be used as the
+        response. The next application down the stack will not be executed and
+        process_response will not be called.
+
+        Otherwise, the next application down the stack will be executed and
+        process_response will be called with the generated response.
+
+        By default this method does not return a value.
+        """
+
+    def process_response(self, response):
+        """Do whatever you'd like to the response.
+
+        By default the response is returned unmodified.
+        """
+        return response
+
+
+class AuthProtocol(_BaseAuthProtocol):
     """Middleware that handles authenticating client calls."""
 
     _SIGNING_CERT_FILE_NAME = 'signing_cert.pem'
     _SIGNING_CA_FILE_NAME = 'cacert.pem'
 
     def __init__(self, app, conf):
-        self._LOG = logging.getLogger(conf.get('log_name', __name__))
-        self._LOG.info(_LI('Starting Keystone auth_token middleware'))
+        log = logging.getLogger(conf.get('log_name', __name__))
+        log.info(_LI('Starting Keystone auth_token middleware'))
+        super(AuthProtocol, self).__init__(app, log=log)
+
         # NOTE(wanghong): If options are set in paste file, all the option
         # values passed into conf are string type. So, we should convert the
         # conf value into correct type.
         self._conf = _conf_values_type_convert(conf)
-        self._app = app
 
         # delay_auth_decision means we still allow unauthenticated requests
         # through and we let the downstream service make the final decision
@@ -438,7 +482,7 @@ class AuthProtocol(object):
 
         self._auth_uri = self._conf_get('auth_uri')
         if not self._auth_uri:
-            self._LOG.warning(
+            self.log.warning(
                 _LW('Configuring auth_uri to point to the public identity '
                     'endpoint is required; clients may not be able to '
                     'authenticate against an admin endpoint'))
@@ -449,7 +493,7 @@ class AuthProtocol(object):
             self._auth_uri = self._identity_server.auth_uri
 
         self._signing_directory = _signing_dir.SigningDirectory(
-            directory_name=self._conf_get('signing_dir'), log=self._LOG)
+            directory_name=self._conf_get('signing_dir'), log=self.log)
 
         self._token_cache = self._token_cache_factory()
 
@@ -459,7 +503,7 @@ class AuthProtocol(object):
                                                      self._signing_directory,
                                                      self._identity_server,
                                                      self._cms_verify,
-                                                     self._LOG)
+                                                     self.log)
 
         self._check_revocations_for_cached = self._conf_get(
             'check_revocations_for_cached')
@@ -471,13 +515,14 @@ class AuthProtocol(object):
         else:
             return CONF[group][name]
 
-    @webob.dec.wsgify(RequestClass=_request._AuthTokenRequest)
-    def __call__(self, request):
-        """Handle incoming request.
-
-        Authenticate send downstream on success. Reject request if
-        we can't authenticate.
+    def process_request(self, request):
+        """Process request.
 
+        Evaluate the headers in a request and attempt to authenticate the
+        request against the identity server. If authenticated then additional
+        headers are added to the request for use by applications. If not
+        authenticated the request will be rejected or marked unauthenticated
+        depending on configuration.
         """
         self._token_cache.initialize(request.environ)
         request.remove_auth_headers()
@@ -487,54 +532,58 @@ class AuthProtocol(object):
             serv_auth_ref = None
 
             try:
-                self._LOG.debug('Authenticating user token')
+                self.log.debug('Authenticating user token')
                 user_token_info = self._get_user_token_from_request(request)
                 user_auth_ref, user_token_info = self._validate_token(
-                    user_token_info, request.environ)
+                    user_token_info, request)
                 request.environ['keystone.token_info'] = user_token_info
                 request.set_user_headers(user_auth_ref,
                                          self._include_service_catalog)
             except exc.InvalidToken:
                 if self._delay_auth_decision:
-                    self._LOG.info(
+                    self.log.info(
                         _LI('Invalid user token - deferring reject '
                             'downstream'))
                     request.headers['X-Identity-Status'] = 'Invalid'
                 else:
-                    self._LOG.info(
+                    self.log.info(
                         _LI('Invalid user token - rejecting request'))
                     self._reject_request()
 
             try:
-                self._LOG.debug('Authenticating service token')
-                serv_token = request.headers.get('X-Service-Token')
-                if serv_token is not None:
+                self.log.debug('Authenticating service token')
+                if request.service_token is not None:
                     serv_auth_ref, serv_token_info = self._validate_token(
-                        serv_token, request.environ)
+                        request.service_token, request)
                     request.set_service_headers(serv_auth_ref)
             except exc.InvalidToken:
                 if self._delay_auth_decision:
-                    self._LOG.info(
+                    self.log.info(
                         _LI('Invalid service token - deferring reject '
                             'downstream'))
                     request.headers['X-Service-Identity-Status'] = 'Invalid'
                 else:
-                    self._LOG.info(
+                    self.log.info(
                         _LI('Invalid service token - rejecting request'))
                     self._reject_request()
 
-            p = _user_plugin.UserAuthPlugin(user_auth_ref, serv_auth_ref)
-            request.environ['keystone.token_auth'] = p
+            request.token_auth = _user_plugin.UserAuthPlugin(user_auth_ref,
+                                                             serv_auth_ref)
 
         except exc.ServiceError as e:
-            self._LOG.critical(_LC('Unable to obtain admin token: %s'), e)
+            self.log.critical(_LC('Unable to obtain admin token: %s'), e)
             raise webob.exc.HTTPServiceUnavailable()
 
-        if self._LOG.isEnabledFor(logging.DEBUG):
-            self._LOG.debug('Received request from %s' % p._log_format)
+        if self.log.isEnabledFor(logging.DEBUG):
+            self.log.debug('Received request from %s' %
+                           request.token_auth._log_format)
 
-        response = request.get_response(self._app)
+    def process_response(self, response):
+        """Process Response.
 
+        Add WWW-Authenticate headers to failed requests so users know where to
+        authenticate for future requests.
+        """
         if response.status_int == 401:
             response.headers.extend(self._reject_auth_headers)
 
@@ -548,13 +597,13 @@ class AuthProtocol(object):
         :raises exc.InvalidToken: if no token is provided in request
 
         """
-        token = request.headers.get('X-Auth-Token',
-                                    request.headers.get('X-Storage-Token'))
+        token = request.user_token
+
         if token:
             return token
         else:
             if not self._delay_auth_decision:
-                self._LOG.debug('Headers: %s', dict(request.headers))
+                self.log.debug('Headers: %s', dict(request.headers))
             raise exc.InvalidToken(_('Unable to find token in headers'))
 
     @property
@@ -608,11 +657,11 @@ class AuthProtocol(object):
             if cached:
                 return cached
 
-    def _validate_token(self, token, env):
+    def _validate_token(self, token, request):
         """Authenticate user token
 
         :param token: token id
-        :param env: wsgi environment
+        :param request: incoming request
         :returns: uncrypted body of the token if the token is valid
         :raises exc.InvalidToken: if token is rejected
 
@@ -644,11 +693,11 @@ class AuthProtocol(object):
                         verified = self._verify_signed_token(token,
                                                              token_hashes)
                 except exceptions.CertificateConfigError:
-                    self._LOG.warning(_LW('Fetch certificate config failed, '
-                                          'fallback to online validation.'))
+                    self.log.warning(_LW('Fetch certificate config failed, '
+                                         'fallback to online validation.'))
                 except exc.RevocationListError:
-                    self._LOG.warning(_LW('Fetch revocation list failed, '
-                                          'fallback to online validation.'))
+                    self.log.warning(_LW('Fetch revocation list failed, '
+                                         'fallback to online validation.'))
 
                 if verified is not None:
                     data = jsonutils.loads(verified)
@@ -672,23 +721,23 @@ class AuthProtocol(object):
                 msg = _('Unable to determine service tenancy.')
                 raise exc.InvalidToken(msg)
 
-            self._confirm_token_bind(data, env)
+            self._confirm_token_bind(auth_ref, request)
 
             if not cached:
                 self._token_cache.store(token_hashes[0], data)
 
             return auth_ref, data
         except (exceptions.ConnectionRefused, exceptions.RequestTimeout):
-            self._LOG.debug('Token validation failure.', exc_info=True)
-            self._LOG.warning(_LW('Authorization failed for token'))
+            self.log.debug('Token validation failure.', exc_info=True)
+            self.log.warning(_LW('Authorization failed for token'))
             raise exc.InvalidToken(_('Token authorization failed'))
         except exc.ServiceError:
             raise
         except Exception:
-            self._LOG.debug('Token validation failure.', exc_info=True)
+            self.log.debug('Token validation failure.', exc_info=True)
             if token_hashes:
                 self._token_cache.store_invalid(token_hashes[0])
-            self._LOG.warning(_LW('Authorization failed for token'))
+            self.log.warning(_LW('Authorization failed for token'))
             raise exc.InvalidToken(_('Token authorization failed'))
 
     def _invalid_user_token(self, msg=False):
@@ -698,17 +747,17 @@ class AuthProtocol(object):
 
         raise exc.InvalidToken(msg)
 
-    def _confirm_token_bind(self, data, env):
+    def _confirm_token_bind(self, auth_ref, req):
         bind_mode = self._conf_get('enforce_token_bind')
 
         if bind_mode == _BIND_MODE.DISABLED:
             return
 
         try:
-            if _token_is_v2(data):
-                bind = data['access']['token']['bind']
-            elif _token_is_v3(data):
-                bind = data['token']['bind']
+            if auth_ref.version == 'v2.0':
+                bind = auth_ref['token']['bind']
+            elif auth_ref.version == 'v3':
+                bind = auth_ref['bind']
             else:
                 self._invalid_user_token()
         except KeyError:
@@ -722,7 +771,7 @@ class AuthProtocol(object):
                 # no bind provided and none required
                 return
             else:
-                self._LOG.info(_LI('No bind information present in token.'))
+                self.log.info(_LI('No bind information present in token.'))
                 self._invalid_user_token()
 
         # get the named mode if bind_mode is not one of the predefined
@@ -732,32 +781,32 @@ class AuthProtocol(object):
             name = bind_mode
 
         if name and name not in bind:
-            self._LOG.info(_LI('Named bind mode %s not in bind information'),
-                           name)
+            self.log.info(_LI('Named bind mode %s not in bind information'),
+                          name)
             self._invalid_user_token()
 
         for bind_type, identifier in six.iteritems(bind):
             if bind_type == _BIND_MODE.KERBEROS:
-                if not env.get('AUTH_TYPE', '').lower() == 'negotiate':
-                    self._LOG.info(_LI('Kerberos credentials required and '
-                                       'not present.'))
+                if req.auth_type != 'negotiate':
+                    self.log.info(_LI('Kerberos credentials required and '
+                                      'not present.'))
                     self._invalid_user_token()
 
-                if not env.get('REMOTE_USER') == identifier:
-                    self._LOG.info(_LI('Kerberos credentials do not match '
-                                       'those in bind.'))
+                if req.remote_user != identifier:
+                    self.log.info(_LI('Kerberos credentials do not match '
+                                      'those in bind.'))
                     self._invalid_user_token()
 
-                self._LOG.debug('Kerberos bind authentication successful.')
+                self.log.debug('Kerberos bind authentication successful.')
 
             elif bind_mode == _BIND_MODE.PERMISSIVE:
-                self._LOG.debug('Ignoring Unknown bind for permissive mode: '
-                                '%(bind_type)s: %(identifier)s.',
-                                {'bind_type': bind_type,
-                                 'identifier': identifier})
+                self.log.debug('Ignoring Unknown bind for permissive mode: '
+                               '%(bind_type)s: %(identifier)s.',
+                               {'bind_type': bind_type,
+                                'identifier': identifier})
 
             else:
-                self._LOG.info(
+                self.log.info(
                     _LI('Couldn`t verify unknown bind: %(bind_type)s: '
                         '%(identifier)s.'),
                     {'bind_type': bind_type, 'identifier': identifier})
@@ -779,7 +828,7 @@ class AuthProtocol(object):
                                       signing_ca_path,
                                       inform=inform).decode('utf-8')
             except cms.subprocess.CalledProcessError as err:
-                self._LOG.warning(_LW('Verify error: %s'), err)
+                self.log.warning(_LW('Verify error: %s'), err)
                 raise
 
         try:
@@ -795,7 +844,7 @@ class AuthProtocol(object):
             except exceptions.CertificateConfigError as err:
                 # if this is still occurring, something else is wrong and we
                 # need err.output to identify the problem
-                self._LOG.error(_LE('CMS Verify output: %s'), err.output)
+                self.log.error(_LE('CMS Verify output: %s'), err.output)
                 raise
 
     def _verify_signed_token(self, signed_text, token_ids):
@@ -848,7 +897,7 @@ class AuthProtocol(object):
         else:
             plugin_class = _auth.AuthTokenPlugin
             # logger object is a required parameter of the default plugin
-            plugin_kwargs['log'] = self._LOG
+            plugin_kwargs['log'] = self.log
 
         plugin_opts = plugin_class.get_options()
         CONF.register_opts(plugin_opts, group=group)
@@ -887,7 +936,7 @@ class AuthProtocol(object):
         if auth_version is not None:
             auth_version = discover.normalize_version_number(auth_version)
         return _identity.IdentityServer(
-            self._LOG,
+            self.log,
             adap,
             include_service_catalog=self._include_service_catalog,
             requested_auth_version=auth_version)
@@ -913,12 +962,12 @@ class AuthProtocol(object):
 
         if security_strategy:
             secret_key = self._conf_get('memcache_secret_key')
-            return _cache.SecureTokenCache(self._LOG,
+            return _cache.SecureTokenCache(self.log,
                                            security_strategy,
                                            secret_key,
                                            **cache_kwargs)
         else:
-            return _cache.TokenCache(self._LOG, **cache_kwargs)
+            return _cache.TokenCache(self.log, **cache_kwargs)
 
 
 def filter_factory(global_conf, **local_conf):
